# Generated by roxygen2: do not edit by hand

S3method("$<-",id_tbl)
S3method("[",id_tbl)
S3method("[<-",id_tbl)
S3method("[[<-",id_tbl)
S3method("dimnames<-",id_tbl)
S3method("names<-",col_cfg)
S3method("names<-",concept)
S3method("names<-",id_cfg)
S3method("names<-",id_tbl)
S3method("names<-",item)
S3method("names<-",tbl_cfg)
S3method("row.names<-",id_tbl)
S3method(aggregate,cncpt)
S3method(aggregate,id_tbl)
S3method(aggregate,rec_cncpt)
S3method(anyDuplicated,id_tbl)
S3method(as.data.frame,id_tbl)
S3method(as.data.table,id_tbl)
S3method(as.list,concept)
S3method(as.list,item)
S3method(as.list,src_env)
S3method(as_col_cfg,col_cfg)
S3method(as_col_cfg,default)
S3method(as_col_cfg,src_cfg)
S3method(as_col_cfg,src_tbl)
S3method(as_concept,cncpt)
S3method(as_concept,concept)
S3method(as_concept,default)
S3method(as_concept,list)
S3method(as_id_cfg,default)
S3method(as_id_cfg,id_cfg)
S3method(as_id_cfg,src_cfg)
S3method(as_id_cfg,src_env)
S3method(as_id_tbl,data.table)
S3method(as_id_tbl,default)
S3method(as_id_tbl,id_tbl)
S3method(as_item,cncpt)
S3method(as_item,concept)
S3method(as_item,default)
S3method(as_item,item)
S3method(as_item,itm)
S3method(as_item,list)
S3method(as_item,rec_cncpt)
S3method(as_ptype,data.frame)
S3method(as_ptype,data.table)
S3method(as_ptype,default)
S3method(as_ptype,id_tbl)
S3method(as_src_cfg,default)
S3method(as_src_cfg,src_cfg)
S3method(as_src_cfg,src_env)
S3method(as_src_env,character)
S3method(as_src_env,default)
S3method(as_src_env,src_env)
S3method(as_src_env,src_tbl)
S3method(as_src_tbl,character)
S3method(as_src_tbl,default)
S3method(as_src_tbl,itm)
S3method(as_src_tbl,src_env)
S3method(as_src_tbl,src_tbl)
S3method(as_tbl_cfg,default)
S3method(as_tbl_cfg,src_cfg)
S3method(as_tbl_cfg,src_tbl)
S3method(as_tbl_cfg,tbl_cfg)
S3method(as_ts_tbl,data.table)
S3method(as_ts_tbl,default)
S3method(as_ts_tbl,id_tbl)
S3method(as_ts_tbl,ts_tbl)
S3method(as_win_tbl,data.table)
S3method(as_win_tbl,default)
S3method(as_win_tbl,id_tbl)
S3method(as_win_tbl,ts_tbl)
S3method(as_win_tbl,win_tbl)
S3method(attach_src,character)
S3method(attach_src,default)
S3method(attach_src,src_cfg)
S3method(change_interval,data.table)
S3method(change_interval,default)
S3method(change_interval,ts_tbl)
S3method(col_renamer,data.table)
S3method(col_renamer,default)
S3method(col_renamer,id_tbl)
S3method(col_renamer,ts_tbl)
S3method(col_renamer,win_tbl)
S3method(default_vars,col_cfg)
S3method(default_vars,src_tbl)
S3method(dimnames,itm)
S3method(do_callback,col_itm)
S3method(do_callback,default)
S3method(do_callback,fun_itm)
S3method(do_callback,hrd_itm)
S3method(do_callback,itm)
S3method(do_callback,rec_cncpt)
S3method(do_itm_load,col_itm)
S3method(do_itm_load,default)
S3method(do_itm_load,fun_itm)
S3method(do_itm_load,hrd_itm)
S3method(do_itm_load,itm)
S3method(do_itm_load,nul_itm)
S3method(downgrade_id,default)
S3method(downgrade_id,id_tbl)
S3method(downgrade_id,ts_tbl)
S3method(download_src,aumc_cfg)
S3method(download_src,character)
S3method(download_src,default)
S3method(download_src,hirid_cfg)
S3method(download_src,src_cfg)
S3method(duplicated,id_tbl)
S3method(dur_var,default)
S3method(dur_var,itm)
S3method(dur_var,win_tbl)
S3method(format,col_cfg)
S3method(format,concept)
S3method(format,id_cfg)
S3method(format,id_tbl)
S3method(format,item)
S3method(format,src_env)
S3method(format,tbl_cfg)
S3method(get_itm_var,default)
S3method(get_itm_var,itm)
S3method(get_target,cncpt)
S3method(get_target,default)
S3method(get_target,itm)
S3method(id_map_helper,default)
S3method(id_map_helper,src_env)
S3method(id_orig_helper,default)
S3method(id_orig_helper,miiv_env)
S3method(id_orig_helper,src_env)
S3method(id_vars,col_cfg)
S3method(id_vars,default)
S3method(id_vars,id_cfg)
S3method(id_vars,id_tbl)
S3method(id_vars,itm)
S3method(id_vars,src_tbl)
S3method(id_win_helper,aumc_env)
S3method(id_win_helper,default)
S3method(id_win_helper,eicu_env)
S3method(id_win_helper,hirid_env)
S3method(id_win_helper,miiv_env)
S3method(id_win_helper,mimic_env)
S3method(id_win_helper,sic_env)
S3method(import_src,aumc_cfg)
S3method(import_src,character)
S3method(import_src,default)
S3method(import_src,src_cfg)
S3method(import_tbl,default)
S3method(import_tbl,tbl_cfg)
S3method(index_var,col_cfg)
S3method(index_var,default)
S3method(index_var,itm)
S3method(index_var,src_tbl)
S3method(index_var,ts_tbl)
S3method(init_cncpt,cncpt)
S3method(init_cncpt,default)
S3method(init_cncpt,fct_cncpt)
S3method(init_cncpt,num_cncpt)
S3method(init_cncpt,rec_cncpt)
S3method(init_cncpt,unt_cncpt)
S3method(init_itm,col_itm)
S3method(init_itm,default)
S3method(init_itm,fun_itm)
S3method(init_itm,hrd_itm)
S3method(init_itm,itm)
S3method(init_itm,rgx_itm)
S3method(init_itm,sel_itm)
S3method(interval,default)
S3method(interval,difftime)
S3method(interval,itm)
S3method(interval,ts_tbl)
S3method(load_concepts,character)
S3method(load_concepts,cncpt)
S3method(load_concepts,concept)
S3method(load_concepts,default)
S3method(load_concepts,fct_cncpt)
S3method(load_concepts,integer)
S3method(load_concepts,item)
S3method(load_concepts,itm)
S3method(load_concepts,lgl_cncpt)
S3method(load_concepts,num_cncpt)
S3method(load_concepts,numeric)
S3method(load_concepts,rec_cncpt)
S3method(load_concepts,unt_cncpt)
S3method(load_difftime,aumc_tbl)
S3method(load_difftime,character)
S3method(load_difftime,default)
S3method(load_difftime,eicu_tbl)
S3method(load_difftime,hirid_tbl)
S3method(load_difftime,miiv_tbl)
S3method(load_difftime,mimic_tbl)
S3method(load_difftime,sic_tbl)
S3method(load_id,character)
S3method(load_id,default)
S3method(load_id,fun_itm)
S3method(load_id,itm)
S3method(load_id,src_tbl)
S3method(load_src,character)
S3method(load_src,default)
S3method(load_src,src_tbl)
S3method(load_ts,character)
S3method(load_ts,default)
S3method(load_ts,fun_itm)
S3method(load_ts,itm)
S3method(load_ts,src_tbl)
S3method(load_win,character)
S3method(load_win,default)
S3method(load_win,fun_itm)
S3method(load_win,itm)
S3method(load_win,src_tbl)
S3method(max,num_cncpt)
S3method(merge,id_tbl)
S3method(meta_vars,default)
S3method(meta_vars,id_tbl)
S3method(meta_vars,itm)
S3method(meta_vars,ts_tbl)
S3method(meta_vars,win_tbl)
S3method(min,num_cncpt)
S3method(n_tick,cncpt)
S3method(n_tick,concept)
S3method(n_tick,default)
S3method(n_tick,item)
S3method(n_tick,itm)
S3method(n_tick,tbl_cfg)
S3method(names,col_cfg)
S3method(names,concept)
S3method(names,id_cfg)
S3method(names,item)
S3method(names,src_env)
S3method(names,tbl_cfg)
S3method(prepare_query,col_itm)
S3method(prepare_query,default)
S3method(prepare_query,hrd_itm)
S3method(prepare_query,rgx_itm)
S3method(prepare_query,sel_itm)
S3method(print,cncpt)
S3method(print,id_tbl)
S3method(print,itm)
S3method(print,src_cfg)
S3method(print,src_env)
S3method(reclass_tbl,"NULL")
S3method(reclass_tbl,data.frame)
S3method(reclass_tbl,data.table)
S3method(reclass_tbl,default)
S3method(reclass_tbl,id_tbl)
S3method(reclass_tbl,ts_tbl)
S3method(reclass_tbl,win_tbl)
S3method(replace_na,data.table)
S3method(replace_na,default)
S3method(replace_na,logical)
S3method(replace_na,numeric)
S3method(row.names,id_tbl)
S3method(set_callback,default)
S3method(set_callback,itm)
S3method(set_target,cncpt)
S3method(set_target,concept)
S3method(set_target,default)
S3method(set_target,item)
S3method(set_target,itm)
S3method(setup_src_data,character)
S3method(setup_src_data,src_cfg)
S3method(setup_src_env,default)
S3method(setup_src_env,src_cfg)
S3method(sort,id_tbl)
S3method(split,id_tbl)
S3method(src_extra_cfg,default)
S3method(src_extra_cfg,src_cfg)
S3method(src_extra_cfg,src_env)
S3method(src_name,cncpt)
S3method(src_name,col_cfg)
S3method(src_name,concept)
S3method(src_name,default)
S3method(src_name,id_cfg)
S3method(src_name,item)
S3method(src_name,itm)
S3method(src_name,src_cfg)
S3method(src_name,src_env)
S3method(src_name,src_tbl)
S3method(src_name,tbl_cfg)
S3method(src_prefix,default)
S3method(src_prefix,src_cfg)
S3method(src_prefix,src_env)
S3method(stay_windows,character)
S3method(stay_windows,default)
S3method(stay_windows,list)
S3method(stay_windows,src_env)
S3method(str,id_tbl)
S3method(str,src_env)
S3method(subset_src,cncpt)
S3method(subset_src,concept)
S3method(subset_src,item)
S3method(tbl_name,col_cfg)
S3method(tbl_name,default)
S3method(tbl_name,fun_itm)
S3method(tbl_name,itm)
S3method(tbl_name,nul_itm)
S3method(tbl_name,src_tbl)
S3method(tbl_name,tbl_cfg)
S3method(tbl_sum,id_tbl)
S3method(tbl_sum,src_tbl)
S3method(tbl_sum,ts_tbl)
S3method(tbl_sum,win_tbl)
S3method(time_vars,col_cfg)
S3method(time_vars,data.frame)
S3method(time_vars,default)
S3method(time_vars,itm)
S3method(time_vars,src_tbl)
S3method(time_vars,win_tbl)
S3method(try_add_vars,cncpt)
S3method(try_add_vars,default)
S3method(try_add_vars,item)
S3method(try_add_vars,itm)
S3method(try_add_vars,rec_cncpt)
S3method(unclass_tbl,data.frame)
S3method(unclass_tbl,default)
S3method(unclass_tbl,id_tbl)
S3method(unclass_tbl,ts_tbl)
S3method(unclass_tbl,win_tbl)
S3method(unique,id_tbl)
S3method(units,num_cncpt)
S3method(upgrade_id,default)
S3method(upgrade_id,id_tbl)
S3method(upgrade_id,ts_tbl)
S3method(validate_tbl,data.frame)
S3method(validate_tbl,default)
S3method(validate_tbl,id_tbl)
S3method(validate_tbl,ts_tbl)
S3method(validate_tbl,win_tbl)
S3method(vec_proxy_compare,id_cfg)
S3method(vec_ptype_abbr,col_cfg)
S3method(vec_ptype_abbr,id_cfg)
S3method(vec_ptype_abbr,tbl_cfg)
S3method(vec_ptype_full,col_cfg)
S3method(vec_ptype_full,id_cfg)
S3method(vec_ptype_full,tbl_cfg)
export(add_concept)
export(add_weight)
export(apply_map)
export(as_col_cfg)
export(as_concept)
export(as_id_cfg)
export(as_id_tbl)
export(as_item)
export(as_ptype)
export(as_src_cfg)
export(as_src_env)
export(as_src_tbl)
export(as_tbl_cfg)
export(as_ts_tbl)
export(as_win_tbl)
export(attach_src)
export(attached_srcs)
export(auto_attach_srcs)
export(avpu)
export(binary_op)
export(bmi)
export(calc_dur)
export(cbind_id_tbl)
export(change_dur_unit)
export(change_id)
export(change_interval)
export(col_renamer)
export(collapse)
export(combine_callbacks)
export(comp_na)
export(concept)
export(concept_availability)
export(config_paths)
export(convert_unit)
export(data)
export(data_col)
export(data_dir)
export(data_var)
export(data_vars)
export(days)
export(default_vars)
export(delta_cummin)
export(delta_min)
export(delta_start)
export(detach_src)
export(do_callback)
export(do_itm_load)
export(downgrade_id)
export(download_src)
export(dt_gforce)
export(dur_col)
export(dur_unit)
export(dur_var)
export(expand)
export(explain_dictionary)
export(fill_gaps)
export(first_elem)
export(fmt_msg)
export(gcs)
export(get_config)
export(get_itm_var)
export(get_target)
export(has_gaps)
export(has_no_gaps)
export(hop)
export(hours)
export(id_col)
export(id_map)
export(id_map_helper)
export(id_orig_helper)
export(id_origin)
export(id_tbl)
export(id_var)
export(id_var_opts)
export(id_vars)
export(id_win_helper)
export(id_windows)
export(import_src)
export(import_tbl)
export(index_col)
export(index_var)
export(init_cncpt)
export(init_itm)
export(interval)
export(is_cncpt)
export(is_concept)
export(is_data_avail)
export(is_false)
export(is_id_tbl)
export(is_item)
export(is_itm)
export(is_regular)
export(is_sorted)
export(is_src_env)
export(is_src_tbl)
export(is_tbl_avail)
export(is_true)
export(is_ts_tbl)
export(is_unique)
export(is_val)
export(is_win_tbl)
export(item)
export(last_elem)
export(load_concepts)
export(load_dictionary)
export(load_difftime)
export(load_id)
export(load_src)
export(load_src_cfg)
export(load_ts)
export(load_win)
export(max_or_na)
export(merge_lst)
export(meta_vars)
export(mews_score)
export(min_or_na)
export(mins)
export(msg_progress)
export(n_tick)
export(new_cncpt)
export(new_concept)
export(new_item)
export(new_itm)
export(new_src_env)
export(new_src_tbl)
export(news_score)
export(norepi_equiv)
export(not_val)
export(pafi)
export(prepare_query)
export(qsofa_score)
export(rbind_id_tbl)
export(rbind_lst)
export(read_psv)
export(reclass_tbl)
export(remove_gaps)
export(rename_cols)
export(replace_na)
export(rm_cols)
export(rm_na)
export(safi)
export(secs)
export(sep3)
export(set_callback)
export(set_config)
export(set_target)
export(set_val)
export(setup_src_data)
export(setup_src_env)
export(sirs_score)
export(slide)
export(slide_index)
export(sofa_cardio)
export(sofa_cns)
export(sofa_coag)
export(sofa_liver)
export(sofa_renal)
export(sofa_resp)
export(sofa_score)
export(src_data_avail)
export(src_data_dir)
export(src_extra_cfg)
export(src_name)
export(src_prefix)
export(src_tbl_avail)
export(stay_windows)
export(subset_src)
export(supp_o2)
export(susp_inf)
export(tbl_name)
export(time_step)
export(time_unit)
export(time_vars)
export(transform_fun)
export(try_add_vars)
export(ts_tbl)
export(unclass_tbl)
export(unmerge)
export(upgrade_id)
export(urine24)
export(validate_tbl)
export(vaso60)
export(vaso_ind)
export(vent_ind)
export(weeks)
export(win_tbl)
export(write_psv)
if (getRversion() >= "4.0.0") { S3method(cbind, id_tbl) }
if (getRversion() >= "4.0.0") { S3method(rbind, id_tbl) }
import(vctrs)
importFrom(assertthat,"on_failure<-")
importFrom(assertthat,are_equal)
importFrom(assertthat,has_attr)
importFrom(assertthat,has_name)
importFrom(assertthat,is.count)
importFrom(assertthat,is.dir)
importFrom(assertthat,is.flag)
importFrom(assertthat,is.number)
importFrom(assertthat,is.scalar)
importFrom(assertthat,is.string)
importFrom(assertthat,see_if)
importFrom(assertthat,validate_that)
importFrom(cli,no)
importFrom(cli,qty)
importFrom(cli,symbol)
importFrom(curl,handle_setopt)
importFrom(curl,new_handle)
importFrom(curl,parse_headers)
importFrom(data.table,"%chin%")
importFrom(data.table,":=")
importFrom(data.table,.EACHI)
importFrom(data.table,.I)
importFrom(data.table,.SD)
importFrom(data.table,as.data.table)
importFrom(data.table,copy)
importFrom(data.table,data.table)
importFrom(data.table,fifelse)
importFrom(data.table,first)
importFrom(data.table,last)
importFrom(data.table,rbindlist)
importFrom(data.table,set)
importFrom(data.table,setDF)
importFrom(data.table,setDT)
importFrom(data.table,setattr)
importFrom(data.table,setcolorder)
importFrom(data.table,setnames)
importFrom(data.table,setorderv)
importFrom(rlang,"!!")
importFrom(rlang,.data)
importFrom(rlang,.env)
importFrom(rlang,as_label)
importFrom(rlang,caller_env)
importFrom(rlang,enexpr)
importFrom(rlang,is_quosure)
importFrom(rlang,is_symbol)
importFrom(rlang,quo_get_env)
importFrom(rlang,quo_get_expr)
importFrom(stats,aggregate)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(tibble,tbl_sum)
importFrom(utils,adist)
importFrom(utils,available.packages)
importFrom(utils,head)
importFrom(utils,install.packages)
importFrom(utils,ls.str)
importFrom(utils,packageDescription)
importFrom(utils,str)
importFrom(utils,tail)
importFrom(utils,unzip)
