\newcommand{\subs}{\ifelse{latex}{\out{\textsubscript{#1}}}{\ifelse{html}{\out{<sub>#1</sub>}}{#1}}}

\newcommand{\sups}{\ifelse{latex}{\out{\textsuperscript{#1}}}{\ifelse{html}{\out{<sup>#1</sup>}}{#1}}}

\newcommand{\tims}{\ifelse{latex}{\out{$\times$}}{\ifelse{html}{\out{&times;}}{x}}}

\newcommand{\leq}{\ifelse{latex}{\out{$\le$}}{\ifelse{html}{\out{&leq;}}{<=}}}

\newcommand{\smu}{\ifelse{latex}{\out{$\mu$}}{\ifelse{html}{\out{&mu;}}{u}}}

\newcommand{\dotm}{\ifelse{latex}{\out{$\cdot$}}{\ifelse{html}{\out{&middot;}}{.}}}
